﻿using SQL_lab_analysis.Classi;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace SQL_lab_analysis
{
    /// <summary>
    /// Logica di interazione per Categories.xaml
    /// </summary>
    public partial class Categories : Window, INotifyPropertyChanged
    {
        public string Access = "Access";
        public string Approach = "Approach";
        public string Solution = "Solution";
        public string Assistance = "Assistance";
        public string Time = "Time";
        private string numSequence;
        public string NumSequence
        {
            get { return numSequence; }
            set { numSequence = value; OnPropertyChanged(); }
        }

        public string CategoryDescription
        {
            set { SubcatDescription.Text = value; }
            get { return SubcatDescription.Text; }
        }

        MainWindow main;
        ObservableCollection<Categoria> categories;
        ObservableCollection<Categoria> categoriesmenu;
        ObservableCollection<Categoria> subCategories;
        public ObservableCollection<Sequenza> sequences;
        ObservableCollection<MyFile> fileInput;
        List<Evento> eventiSingoli;
        private Categoria rootCat;

        public event PropertyChangedEventHandler PropertyChanged;

        public Categories(ObservableCollection<MyFile> myfiles, List<Evento> eventi)
        {
            InitializeComponent();

            DataContext = this;

            categories = new ObservableCollection<Categoria>();
            subCategories = new ObservableCollection<Categoria>();
            sequences = new ObservableCollection<Sequenza>();
            categoriesmenu = new ObservableCollection<Categoria>();
            fileInput = new ObservableCollection<MyFile>();
            NumSequence = "Sequence";

            eventiSingoli = eventi;

            SubCatControl.ItemsSource = subCategories;
            PathControl.ItemsSource = categoriesmenu;
            main = Application.Current.Windows[0] as MainWindow;
            Sequences.ItemsSource = sequences;

            List<Categoria> list = LabInfo.readCategorie();
            foreach (Categoria c in list)
            {
                categories.Add(c);
            }


            rootCat = list[0];
            categoriesmenu.Add(rootCat);

            foreach (Categoria cat in rootCat.SottoCategorie)
                subCategories.Add(cat);

            fileInput = myfiles;
            Files.ItemsSource = fileInput;
            Files.SelectedIndex = 0;

        }

        public Categories(List<Sequenza> sequenza)
        {
            InitializeComponent();
            main = Application.Current.Windows[0] as MainWindow;
            DataContext = this;

            sequences = new ObservableCollection<Sequenza>(sequenza);
            Sequences.ItemsSource = sequences;
            SearchPanelBig.Visibility = Visibility.Hidden;
            JoinBtn.Visibility = Visibility.Hidden;
        }

        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private Categoria findCategoria(string wanted)
        {
            Categoria cat = new Categoria();
            // parto sempre dalla radice

            Categoria menulast = categoriesmenu.Last();

            foreach (Categoria categoria in menulast.SottoCategorie)
                if (categoria.Name == wanted)
                    cat = categoria;

            return cat;
        }


        private void Chip_Click(object sender, RoutedEventArgs e)
        {
            Console.WriteLine("");
            MaterialDesignThemes.Wpf.Chip chip = ((MaterialDesignThemes.Wpf.Chip)sender);
            string category = chip.Content.ToString();
            string dir = chip.DeleteToolTip.ToString();
            SubcatDescription.Text = File.ReadAllText(dir + "\\description.txt");
            Categoria currentCat = findCategoria(category);
            categoriesmenu.Add(currentCat);

            AddSubCategories(currentCat);

        }



        private void AddSubCategories(Categoria currentCat)
        {
            subCategories.Clear();
            foreach (Categoria curr in currentCat.SottoCategorie)
                subCategories.Add(curr);

            Console.WriteLine("stop");

            setSequences(currentCat);
        }


        private void setSequences(Categoria currentCat)
        {
            if (currentCat.padre == null)
                return;
            List<Sequenza> temp = LabInfo.Risultati.readSequences(fileInput[Files.SelectedIndex].CompletePath);
            sequences.Clear();

            try
            {
                List<Sequenza> ris = GestioneCategorie.GetSequences(this, currentCat, temp);

                foreach (Sequenza s in ris)
                    sequences.Add(s);
            }
            catch(Exception e)
            {
                main.setTotalExercises();
                Close();
            }


            NumSequence = "Sequences (" + sequences.Count + " matches)";
        }


        private void PathItem_MouseUp(object sender, MouseButtonEventArgs e)
        {
            TextBlock textb = (TextBlock)sender;
            string catName = textb.Text;
            List<Categoria> temp = new List<Categoria>();

            for (int i = 0; i < categoriesmenu.Count; i++)
            {
                if (categoriesmenu[i].Name == catName)
                {

                    for (int j = 0; j <= i; j++)
                        temp.Add(categoriesmenu[j]);


                }
            }
            categoriesmenu.Clear();
            sequences.Clear();
            SubcatDescription.Text = "";
            foreach (Categoria categoria in temp)
                categoriesmenu.Add(categoria);

            AddSubCategories(temp.Last());
        }


        private void Sequences_SizeChanged(object sender, SizeChangedEventArgs e)
        {

            ListView listView = sender as ListView;
            GridView gView = listView.View as GridView;

            var workingWidth = listView.ActualWidth - SystemParameters.VerticalScrollBarWidth; // take into account vertical scrollbar
            var col1 = 0.70;
            var col2 = 0.20;
            var col3 = 0.15;

            gView.Columns[0].Width = workingWidth * col1;
            gView.Columns[1].Width = workingWidth * col2;
            gView.Columns[1].Width = workingWidth * col3;

        }

        private void Files_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int index = Files.SelectedIndex;
            MyFile file = fileInput[index];
            string path = file.CompletePath;

            Categoria currCat = categoriesmenu.Last();
            /*List<Sequenza> seq = LabInfo.Risultati.readSequences(path);
            sequences.Clear();
            foreach (Sequenza s in seq)
                sequences.Add(s);

            NumSequence = "Sequences (" + sequences.Count + " matches)"; */
            setSequences(currCat);

            CurrentFile.Text = "Current: " + file.FileName;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            ObservableCollection<Categoria> cat = new ObservableCollection<Categoria>();

            // prendo solo le sottocategorie delle macro categorie
            foreach (Categoria c in categories) // categories
                foreach (Categoria s in c.SottoCategorie) // Access, Approach...
                    foreach (Categoria s1 in s.SottoCategorie)
                    {

                        cat.Add(s1);

                    }

            SelectJoinCategories select = new SelectJoinCategories(cat, fileInput);
            select.Show();
        }

        private void StackPanel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (IsSeachEqual.IsChecked == true)
            {
                RicercaSottosequenza r = new RicercaSottosequenza(this, eventiSingoli, true);
                r.Show();
            }
            else
            {
                RicercaSottosequenza r = new RicercaSottosequenza(this, eventiSingoli, false);
                r.Show();
            }
        }


        private void showStudentsId_MouseDown(object sender, MouseButtonEventArgs e)
        {
            StackPanel b = sender as StackPanel;

            var child = VisualTreeHelper.GetChild(b, 0);

            TextBlock count = child as TextBlock;
            int cnt = int.Parse(count.Text.ToString());


            Sequenza wanted = sequences[cnt];

            List<string> studentsIds = main.labInfo.getStudentsMatchSequence(wanted);

            StudentsMatchSequence stud = new StudentsMatchSequence(studentsIds, wanted);
            stud.Show();
        }
    }

}
