﻿using SQL_lab_analysis.Classi;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace SQL_lab_analysis
{
    /// <summary>
    /// Logica di interazione per JoinCategories.xaml
    /// </summary>
    public partial class JoinCategories : Window, INotifyPropertyChanged
    {
        ObservableCollection<StringViewer> showCat;
        ObservableCollection<Categoria> categories;
        ObservableCollection<MyFile> files;
        List<Sequenza> seqFile;
        ObservableCollection<Sequenza> sequences;
        MainWindow main;

        public event PropertyChangedEventHandler PropertyChanged;

        private string numseq;
        public string NumSequence { get { return numseq; } set { numseq = value; OnPropertyChanged(); } }

        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        public JoinCategories(ObservableCollection<Categoria> categ, ObservableCollection<MyFile> Files)
        {
            InitializeComponent();

            DataContext = this;

            showCat = new ObservableCollection<StringViewer>();
            files = new ObservableCollection<MyFile>();
            seqFile = new List<Sequenza>();
            sequences = new ObservableCollection<Sequenza>();
            main = new MainWindow();
            showCat.Add(new StringViewer("Common"));

            categories = categ;

            //foreach (Categoria cat in categ)
             //   showCat.Add(new StringViewer("Only " + cat.FullName));

            files = Files;
            SubCatControl.ItemsSource = showCat;
            FileList.ItemsSource = files;
            FileList.SelectedIndex = 0;
            Sequences.ItemsSource = sequences;
        }

        private void Sequences_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            NumSequence = "Sequences (" + sequences.Count + ")";

            ListView listView = sender as ListView;
            GridView gView = listView.View as GridView;

            var workingWidth = listView.ActualWidth - SystemParameters.VerticalScrollBarWidth; // take into account vertical scrollbar
            var col1 = 0.70;
            var col2 = 0.20;

            gView.Columns[0].Width = workingWidth * col1;
            gView.Columns[1].Width = workingWidth * col2;

        }

        private void Files_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int index = FileList.SelectedIndex;
            MyFile file = files[index];
            CurrentFile.Text = file.FileName;
            string path = file.CompletePath;
            seqFile = LabInfo.Risultati.readSequences(path);
        }

        private void Chip_Click(object sender, RoutedEventArgs e)
        {
            MaterialDesignThemes.Wpf.Chip chip = ((MaterialDesignThemes.Wpf.Chip)sender);
            string content = chip.Content.ToString();

            sequences.Clear();
            if (content == "Common")
            {
                List<List<Sequenza>> sequenzeCategorie = new List<List<Sequenza>>();
                List<Sequenza> risultato = new List<Sequenza>();

                try
                {
                    foreach (Categoria cat in categories)
                    {
                        List<Sequenza> seq = GestioneCategorie.GetSequences(null, cat, seqFile);
                        sequenzeCategorie.Add(seq);
                    }
                }
                catch(Exception e1)
                {
                    Console.WriteLine(e1.Message);
                    main.setTotalExercises();
                    Close();
                }
                Console.WriteLine("Primo passo.");


                List<Sequenza> selezionata = sequenzeCategorie[0]; // prendo la prima categoria
                int countMatch = 1; // la prima già contiene la sequenza

                int oldCountMatch;
                foreach (Sequenza s in selezionata)
                {
                    oldCountMatch = countMatch = 1; // faccio ripartire il conteggio per la nuova sequenza

                    for (int j = 1; j < sequenzeCategorie.Count; j++)
                    {
                        List<Sequenza> daConfrontare = sequenzeCategorie[j]; // estraggo le sequenze della categoria successiva
                        foreach (Sequenza s1 in daConfrontare)
                        {
                            if (s1.Equals(s))
                            {
                                countMatch++; // ho trovato un match - posso passare alla categoria successiva
                                break;
                            }
                        }
                        if (countMatch == oldCountMatch) // non ho trovato un match - posso passare alla prossima sequenza di selezionata
                            // perchè la ricercata s deve essere presente in tutte le sequenze!!
                            break;
                    }
                    if (countMatch == sequenzeCategorie.Count) // la sequenza è presente in tutte le categorie
                        risultato.Add(s);
                }

                foreach (Sequenza sequenza in risultato)
                    sequences.Add(sequenza);

            }
            NumSequence = "Sequences (" + sequences.Count + ")";
        }
    }
}
