﻿using SQL_lab_analysis.Classi;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace SQL_lab_analysis
{
    /// <summary>
    /// Logica di interazione per Results.xaml
    /// </summary>
    public partial class Results : Window, INotifyPropertyChanged
    {
        public String Path { get; set; }
        String[] results;

        private ObservableCollection<Sequenza> sequences;
        public ObservableCollection<MyFile> fileNames;
        public bool closing = false;
        MainWindow main;
        Categories categories;

        // ricerca
        public Sequenza Ricercata { get; set; } // viene riempita dalla ricerca
        private List<Evento> eventiSingoli;
        public List<Evento> Eventi
        {
            get { return eventiSingoli; }
            set
            {
                if (eventiSingoli == value) return;
                eventiSingoli = value;
                OnPropertyChanged();
            }
        }
        private string numSequence;

        public string NumSequence
        {
            get { return numSequence; }
            set { numSequence = value; OnPropertyChanged(); }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public ObservableCollection<Sequenza> Sequences
        {
            get { return sequences; }
            set
            {
                if (sequences == value) return;
                sequences = value;
                OnPropertyChanged();
            }
        }

        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        public Results(String path)
        {
            InitializeComponent();
            Path = path;
            DataContext = this;
            sequences = new ObservableCollection<Sequenza>();
            resultsList.ItemsSource = sequences;
            main = Application.Current.Windows[0] as MainWindow;
            fileNames = new ObservableCollection<MyFile>();
            FileControl.ItemsSource = fileNames;
            foreach (MyFile f in main.FileNames)
                fileNames.Add(f);

            var coll = fileNames.OrderBy(o => o.FileName);
            MyFile[] vett = coll.ToArray<MyFile>();
            fileNames.Clear();
            foreach (MyFile file in vett)
                fileNames.Add(file);

            eventiSingoli = new List<Evento>();

            printRes(fileNames[0].CompletePath);
        }

        public Results(List<Sequenza> sequenza)
        {
            InitializeComponent();
            main = Application.Current.Windows[0] as MainWindow;
            DataContext = this;

            eventiSingoli = new List<Evento>();

            sequences = new ObservableCollection<Sequenza>(sequenza);


            resultsList.ItemsSource = sequences;
            SearchPanelTot.Visibility = Visibility.Hidden;
            FileNames.Visibility = Visibility.Collapsed;
            Parameters.Visibility = Visibility.Collapsed;
            ButtonCat.Visibility = Visibility.Collapsed;
        }

        public void printRes(string currentPath)
        {
            HeaderResult header;
            try
            {

                header = LabInfo.Risultati.getHeader(currentPath);
                results = File.ReadAllLines(currentPath);

                if (header.Mingap != 0)
                    Mingap.Text = "Mingap: " + header.Mingap;
                if (header.Maxgap != 0)
                    Maxgap.Text = "Maxgap: " + header.Maxgap;
                if (header.MaxWindow != 0)
                    Window.Text = "Maxwindow: " + header.MaxWindow;
                if (header.TotalStudent != 0)
                    TotStud.Text = "# students: " + header.TotalStudent;
                if (header.Minsup != 0 && header.MinsupCount != 0)
                    Minsup.Text = "Minsup: " + header.Minsup + " (" + header.MinsupCount + ")";

                eventiSingoli.Clear();
                List<Sequenza> lis = new List<Sequenza>();
                try
                {
                    lis = LabInfo.Risultati.readSequences(currentPath);

                }
                catch
                {
                    main.setTotalExercises();
                    this.Close();
                    closing = true; 
                    return;
                }
                sequences.Clear();
                foreach (Sequenza s in lis)
                {
                    if (s.Lunghezza == 1)
                        eventiSingoli.Add(s.getFirstEvento());
                    sequences.Add(s);

                }

                NumSequence = "Sequences (" + sequences.Count + ")";

                Console.WriteLine("Sequences ended.");
            }
            catch (FileNotFoundException e)
            {
                e.ToString();
            }
        }



        private Sequenza parseSequenza(string seq)
        {
            string[] ress = seq.Split(new string[] { "->" }, StringSplitOptions.None);

            for (int i = 0; i < ress.Length; i++)
                ress[i] = ress[i].Trim();

            Sequenza sequenza = new Sequenza();
            sequenza.SeqString = seq;


            for (int j = 0; j < ress.Length; j++)
            {
                // creazione evento
                int eserc = int.Parse(ress[j].Substring(1, 2));
                int tipo = int.Parse(ress[j].Substring(0, 1));
                Evento e = new Evento(eserc, tipo);
                sequenza.add(e);
            }
            return sequenza;
        }

        private void Chip_Click(object sender, RoutedEventArgs e)
        {
            Console.WriteLine("");
            MaterialDesignThemes.Wpf.Chip chip = ((MaterialDesignThemes.Wpf.Chip)sender);
            string file = chip.Content.ToString();
            string dir = chip.ToolTip.ToString();
            string path = dir + "\\" + file;

            OutputFile.Text = file;
            sequences.Clear();
            clearParameters();
            printRes(path);
        }

        private void clearParameters()
        {
            Mingap.Text = "";
            Maxgap.Text = "";
            Window.Text = "";
            TotStud.Text = "";
            Minsup.Text = "";

        }

        private void StackPanel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (IsSeachEqual.IsChecked == true)
            {
                RicercaSottosequenza r = new RicercaSottosequenza(this, eventiSingoli, true);
                r.Show();
            }
            else
            {
                RicercaSottosequenza r = new RicercaSottosequenza(this, eventiSingoli, false);
                r.Show();
            }

        }

        private void showStudentsId_MouseDown(object sender, MouseButtonEventArgs e)
        {
            StackPanel b = sender as StackPanel;

            var child = VisualTreeHelper.GetChild(b, 0);

            TextBlock count = child as TextBlock;
            int cnt = int.Parse(count.Text.ToString());


            Sequenza wanted = sequences[cnt];

            List<string> studentsIds = main.labInfo.getStudentsMatchSequence(wanted);

            StudentsMatchSequence stud = new StudentsMatchSequence(studentsIds, wanted);
            stud.Show();
        }

        private void Categories_click(object sender, RoutedEventArgs e)
        {
            categories = new Categories(fileNames, eventiSingoli);
            categories.Show();
        }


    }
}
