﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace SQL_lab_analysis
{
    /// <summary>
    /// Logica di interazione per StudentsSequence.xaml
    /// </summary>
    public partial class StudentsSequence : Window, INotifyPropertyChanged
    {
        MainWindow main;
        private ObservableCollection<Studente> studenti;
        private ObservableCollection<Evento> eventi;
        private Studente selected;


        public ObservableCollection<Studente> Studenti
        {
            get { return studenti; }
            set
            {
                studenti = value; OnPropertyChanged();
            }
        }

        public ObservableCollection<Evento> Eventi
        {
            get { return eventi; }
            set
            {
                eventi = value; OnPropertyChanged();
            }
        }

        public StudentsSequence()
        {
            InitializeComponent();
            main = Application.Current.Windows[0] as MainWindow;

            studenti = new ObservableCollection<Studente>(main.labInfo.getAllStudents());
            eventi = new ObservableCollection<Evento>();
            students.ItemsSource = studenti;
            Sequence.ItemsSource = eventi;
            Console.WriteLine("Done.");
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private void students_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListView list = sender as ListView;
            int index = list.SelectedIndex;
            selected = studenti[index];

            Id.Text = "Student's ID: " + selected.Id;

            eventi.Clear();

            foreach (Evento eve in selected.Events)
                eventi.Add(eve);

            countAccess();
            countCorrect();
            countError();
            averageErrorPerEx();

            NAccess.Text = "Number of access: " + selected.numaccess;
            NCorrect.Text = "Number of correct exercises: " + selected.numcorrect;
            NErr.Text = "Number of ex. with errors: " + selected.numerror;
            AvgErr.Text = "Average errors per ex: " + selected.avgErrorPerEx;
        }

        private void countAccess()
        {
            bool[] access = new bool[13];

            foreach (Evento ev in selected.Events)
            {
                if (ev.Event == Evento.Accesso)
                    access[ev.NumEsercizio - 1] = true;
            }
            int cnt = 0;
            for (int i = 0; i < 13; i++)
                if (access[i] == true)
                    cnt++;
            selected.numaccess = cnt;
        }

        private void countCorrect()
        {
            bool[] correct = new bool[13];

            foreach (Evento ev in selected.Events)
            {
                if (ev.Event == Evento.Corretto)
                    correct[ev.NumEsercizio - 1] = true;

            }
            int cnt = 0;
            for (int i = 0; i < 13; i++)
                if (correct[i] == true)
                    cnt++;
            selected.numcorrect = cnt;
        }

        private void countError()
        {
            bool[] error = new bool[13];

            foreach (Evento ev in selected.Events)
            {
                if (ev.Event == Evento.Errore)
                    error[ev.NumEsercizio - 1] = true;
            }
            int cnt = 0;
            for (int i = 0; i < 13; i++)
                if (error[i] == true)
                    cnt++;
            selected.numerror = cnt;
        }

        private void averageErrorPerEx()
        {
            int[] errors = new int[13];

            foreach (Evento ev in selected.Events)
            {
                if (ev.Event == Evento.Errore)
                    errors[ev.NumEsercizio - 1]++;
            }

            int sum = 0;
            for (int cnt = 0; cnt < selected.numaccess; cnt++)
                sum += errors[cnt];

            float avg = (float)sum / selected.numaccess;
            selected.avgErrorPerEx = Math.Round((double)avg, 1);
        }
    }
}
