﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;
using SQL_lab_analysis.Classi;
using System.Globalization;

namespace SQL_lab_analysis
{
    public class LabInfo
    {
        public List<Turno> turni;
       Dictionary<string, Studente> students;
        public static int TotalStudent { get; set; }
        public LabInfo()
        {
            turni = new List<Turno>();

            //initTurni();
            //readTurni();
            students = new Dictionary<string, Studente>();
            readStudents();

            countOnlyAccessExOne();

            //provaSequence();
            
        }

        public class Risultati
        {


            public static HeaderResult getHeader(string currentPath)
            {
                string[] results = File.ReadAllLines(currentPath);
                double minsup = 0, mingap = 0, maxgap = 0, window = 0;
                HeaderResult header = new HeaderResult();
                int minsupcount = 0;

                int i = 0;

                do
                {
                    if (i == 1)
                    {
                        string s = results[i];
                        string name = s.Split(':')[1].Trim();
                    }
                    i++;
                } while (!results[i].Contains("Parameters") && i != results.Length);


                if(i == results.Length)
                {
                    return header; 
                }

                while (!string.IsNullOrEmpty(results[i]))
                {
                    string s = results[i];
                    string[] values = s.Split(':');

                    if (values[0] == "Minsup")
                        minsup = Double.Parse(values[1], CultureInfo.InvariantCulture);


                    if (values[0] == "Mingap")
                        mingap = Double.Parse(values[1]);


                    if (values[0] == "Maxgap")
                        maxgap = Double.Parse(values[1]);


                    if (values[0] == "Maxwindow")
                        window = Double.Parse(values[1]);

                    i++;
                }

                i++; // salto la riga vuota

                string sup = results[i];
                string[] fields = sup.Split(' ');
                minsupcount = int.Parse(fields[1]);
                int seqcount = int.Parse(fields[4]);


                header.Minsup = minsup;
                header.Mingap = mingap;
                header.Maxgap = maxgap;
                header.MaxWindow = window;
                header.MinsupCount = minsupcount;
                header.TotalStudent = seqcount;
                TotalStudent = seqcount;

                return header;
            }

            public static List<Sequenza> readSequences(string currentPath)
            {
                string[] results = File.ReadAllLines(currentPath);
                int i = 0;
                List<Sequenza> ret = new List<Sequenza>();
                int cnt = 1;

                // salto header
                do
                {
                    i++;
                } while (!results[i].Contains("MINSUPPORT"));

                while (i < results.Length)
                {
                    string[] ress = results[i].Split(new string[] { "--" }, StringSplitOptions.None);
                    if (ress.Length == 1)
                    {
                        i++;
                        continue;
                    }
                    string sequenza = ress[0];

                    Sequenza seq = parseSequenza(sequenza.Trim());
                    int count = numCountFreq(ress[1].Trim()); // frequenza della nuova sequenza

                    seq.Frequenza = count;
                    seq.seqId = cnt;

                    seq.Percentuale = Math.Round((double)seq.Frequenza / TotalStudent * 100, 1);

                    if(AllAccess(seq))
                        ret.Add(seq);
                    i++;
                    cnt++;

                }

                var newL = ret.OrderByDescending(s => s.Frequenza);

                List<Sequenza> ret2 = new List<Sequenza>();

                int sequenceId= 0; 
                foreach (Sequenza sequenza in newL)
                {
                    sequenza.seqId = sequenceId++;
                    ret2.Add(sequenza);
                }

                return ret2;
            }

            private static bool AllAccess(Sequenza seq)
            {
                List<Evento> eventi = seq.getEvents();
                bool[] access = new bool[13];
                bool good = true;
                foreach (Evento e in eventi)
                {
                    if (e.Event == "Accesso")
                        access[e.NumEsercizio - 1] = true;
                    else
                    {
                        if (access[e.NumEsercizio - 1] == false && seq.getLunghezza() > 1)
                        {
                            good = false;
                            break;
                        }
                    }
                }
                return good;
                
            }



            private static Sequenza parseSequenza(string seq)
            {
                string[] ress = seq.Split(new string[] { "->" }, StringSplitOptions.None);

                for (int i = 0; i < ress.Length; i++)
                    ress[i] = ress[i].Trim();

                Sequenza sequenza = new Sequenza();
                sequenza.SeqString = seq;


                for (int j = 0; j < ress.Length; j++)
                {
                    // creazione evento
                    int eserc = int.Parse(ress[j].Substring(1, 2));
                    int tipo = int.Parse(ress[j].Substring(0, 1));
                    Evento e = new Evento(eserc, tipo);
                    sequenza.add(e);

                }

                

                return sequenza;
            }

            private static int numCountFreq(string count)
            {
                string[] s = count.Split(' ');
                int res = int.Parse(s[0]);
                return res;
            }
        }

            private void initTurni()
        {
            Turno turno1 = new Turno(new DateTime(1509957000), new DateTime(1509962400));
            Turno turno2 = new Turno(new DateTime(1509962400), new DateTime(1509967800));
            Turno turno3 = new Turno(new DateTime(1510237800), new DateTime(1510243200));
            Turno turno4 = new Turno(new DateTime(1510243200), new DateTime(1510248600));
            Turno turno5 = new Turno(new DateTime(1510248600), new DateTime(1510254000));
            Turno turno6 = new Turno(new DateTime(1510313400), new DateTime(1510318800));
            turno1.Num = 1;
            turno2.Num = 2;
            turno3.Num = 3;
            turno4.Num = 4;
            turno5.Num = 5;
            turno6.Num = 6;
            turni.Add(turno1);
            turni.Add(turno2);
            turni.Add(turno3);
            turni.Add(turno4);
            turni.Add(turno5);
            turni.Add(turno6);
        }

        private void readStudents()
        {
            string filename = FilePath.Lab;
            string[] lines = File.ReadAllLines(filename);


            foreach (string s in lines)
            {
                string[] fields = s.Split(' ');
                string studentid = fields[0];
                string timestamp = fields[1];
                string evento = fields[3];

                Evento ev = createEvent(evento, timestamp);

                

              
                Studente stud = new Studente();
                stud.Id = studentid;

                Studente s1;
                if (!students.TryGetValue(studentid, out s1))
                    students.Add(studentid, stud);
                else
                    stud = s1;
                stud.AddEvent(ev);

            }
            TotalStudent = students.Count;
            Console.Write("Done.");
        }

        /*
        private void readTurni()
        {
            readTurno(1);
            readTurno(2);
            readTurno(3);
            readTurno(4);
            readTurno(5);
            readTurno(6);
            Console.WriteLine("Turno 1: " + turni[0].studenti.Count);
            Console.WriteLine("Turno 2: " + turni[1].studenti.Count);
            Console.WriteLine("Turno 3: " + turni[2].studenti.Count);
            Console.WriteLine("Turno 4: " + turni[3].studenti.Count);
            Console.WriteLine("Turno 5: " + turni[4].studenti.Count);
            Console.WriteLine("Turno 6: " + turni[5].studenti.Count);
        }

        private void readTurno(int turno)
        {
            string filename = "..\\..\\fileInput\\turno" + turno + ".data";
            string[] lines = File.ReadAllLines(filename);

                
            foreach (string s in lines)
            {
                string[] fields = s.Split(' ');
                string studentid = fields[0];
                string timestamp = fields[1];
                string evento = fields[3];

                Evento ev = createEvent(evento, timestamp);


                Turno t = turni[turno - 1];

                if (t == null)
                    return;

                Studente stud = new Studente();
                stud.Id = studentid;


                if (!t.containsStudent(studentid))
                    t.addStudent(stud);
                else
                    stud = t.getStudent(studentid);

                stud.AddEvent(ev);

            }

          
        }
        */
        public List<Sequenza> getAllStudentSequences()
        {
            List<Sequenza> ret = new List<Sequenza>();

            foreach (Turno t in turni)
            {
                foreach (Studente s in t.studenti)
                    ret.Add(s.Sequence);
            }

            return ret;
        }

        public List<Studente> getAllStudents()
        {
            List<Studente> ret = new List<Studente>();

            
              foreach (Studente s in students.Values)
                 ret.Add(s);
            

            return ret;
        }

        public Dictionary<string, Studente> getAllStudentsMap()
        {
            return students; 
        }


        private Turno findTurn(string time)
        {
            Turno ret = null;
            foreach (Turno t in turni)
                if (t.isTimestampInTurno(time))
                    ret = t;

            return ret;
        }

        private Evento createEvent(string evento, string time)
        {
            int eserc = int.Parse(evento.Substring(1, 2));
            int tipo = int.Parse(evento.Substring(0, 1));
            Evento e = new Evento(eserc, tipo);
            e.setTimestamp(time);

            return e; 
        }

        public List<string> getStudentsMatchSequence(Sequenza sequenza)
        {
            List<string> studentsid = new List<string>();

            
            foreach(Studente stud in students.Values)
            {
                Sequenza seq = stud.Sequence;
                if (seq.Contains(sequenza))
                    studentsid.Add(stud.Id);
            }
            

            return studentsid;
        }

        public void countOnlyAccessExOne()
        {
            int cnt = 0; 
            Sequenza sequenza = new Sequenza();
            sequenza.add(new Evento(1, 1)); // solo accesso all'esercizio 1
            foreach (Turno t in turni)
            {
                foreach (Studente stud in t.studenti)
                {
                    Sequenza seq = stud.Sequence;
                    if (seq.Equals(sequenza))
                        cnt++;
                }
            }
            Console.WriteLine("Ci sono " + cnt + " studenti che hanno solo fatto accesso all'esercizio 1.");
        }

        private void provaSequence()
        {
            Sequenza s = new Sequenza();
            s.add(new Evento(5, 1));
            s.add(new Evento(5, 2));
            s.add(new Evento(5, 2));
            s.add(new Evento(5, 2));
            s.add(new Evento(5, 6));
            Console.WriteLine(getStudentsMatchSequence(s).Count);
        }

        public static List<Categoria> readCategorie()
        {
            List<Categoria> categories = new List<Categoria>(); 
            string categoriesPath = FilePath.Utilities;
            string[] directories = Directory.GetDirectories(categoriesPath);
            foreach (string s in directories)
            {
                Categoria cat = readCategoria(s);
                categories.Add(cat);
            }

            return categories;
        }
        public static Categoria readCategoria(string categoria)
        {
            Categoria ret = new Categoria(); 
            Console.WriteLine(categoria);
            
            string[] fields = categoria.Split('\\');
            ret.Name = fields[fields.Length-1];
            ret.Folder = categoria;

            string description = File.ReadAllText(categoria + "\\description.txt");
            ret.Description = description;

            string[] directories = Directory.GetDirectories(categoria);
            if (directories.Length > 0)
            {
                foreach(string dir in directories)
                {
                    Categoria sub = readCategoria(dir);
                    sub.padre = ret;
                    ret.AddSubCategory(sub);
                }
                    
            }

            return ret;
        }

    }
}
