#!/bin/bash


PARSED_OPTIONS=$(getopt -n "$0" -o hs:i:r:u:l:w:d: --long "help,minsup:,input:,result:,maxgap:,mingap:,maxwindow:,dir:" -- "$@")
if [ $? -ne 0 ]
then 
	exit 1
fi

eval set --"$PARSED_OPTIONS"

while true;
do
case "$1" in
	-h|--help)
		echo
		echo "-i, --input: filename - file to be analized. It has to be in the format:"
		echo "-s, --minsup: minimum support of the item to be consider frequent, i.e. percentage of sequences in which the item/itemset has to show up."
		echo "-u, --maxgap: maxgap allowed between itemset within the same sequence (eid/time)."
		echo "-l, --mingap: mingap allowed between itemset within the same sequence (eid/time)."
		echo "-w, --maxwindow: max time window of a subsequence"
		echo "-d, --dir: directory where to save output inside 'results' folder. If not specified, results are saved in the 'results' folder."
	exit -1
	shift;;
	-s|--minsup)
		echo "Minsup: $2"
		minsup=$2
	shift 2;;
	-i|--input)
		echo "File input: $2"
		fileinput=$2
	shift 2;;
	-r|--result)
		echo "File output: $2"
		result=$2
	shift 2;;	
	-u|--maxgap):
		echo "Maxgap: $2"
		maxgap=$2
	shift 2;;
	-l|--mingap):
		echo "Mingap: $2"
		mingap=$2
	shift 2;;
	-w|--maxwindow):
		echo "Maxwindows: $2"
		maxwindow=$2
	shift 2;;
	-d|--dir):
		echo "Directory: $2"
		directory=$2
	shift 2;;
	--)
	break;;	
	
esac
done

if [ -z "$minsup" ] || [ -z "$fileinput" ] #check if parameters are set
then
	echo
	echo "Wrong arguments."
	echo "Mandatory:"
	echo "-i, --input: filename - file to be analized. It has to be in the format:"
	echo "-s, --minsup: minimum support of the item to be consider frequent, i.e. percentage of sequences in which the item/itemset has to show up."
	
	echo "Optional:"
	echo "-u, --maxgap: maxgap allowed between itemset within the same sequence (eid/time)."
	echo "-l, --mingap: mingap allowed between itemset within the same sequence (eid/time)."
 	echo "-w, --maxwindow: max time window of a subsequence."
	echo "-d, --dir: directory where to save output inside 'results' folder. You can also specify a path, like 'dir/subdir/subsubdir'. If not specified, results are saved in the 'results' folder."
   echo "example: ./cspade.sh -i filename -s minsup"
   exit -1
fi

if [ -n "$directory" ]
then	# output dir set
	mydir=results/$directory
	if [ ! -d $mydir ]
	then # create directory
		mkdir -p $mydir
		
	fi
	if [ ! -z "$result"]
	then # output file set. Concat to folder			
		echo "Current result:$result."		
		result=$directory/$result
		echo "Output file in $result"
	else
		cd $mydir
		countfile=$(ls|wc -l) # numer of file in folder, concat it
		countfile=$((countfile+1))
		printf -v countfile "%02d" $countfile
		result="ris$countfile"
		result=$directory/$result
		cd -
	fi
else # directory not set - put only file name
	cd results	
	countfile=$(ls|wc -l)
	echo "Number of file: $countfile"
	result="ris$countfile"
	cd -
fi

cp ./inputs/$fileinput ./TPOSE # input folder for the data 
cd ./TPOSE
./makebin $fileinput out.data
./getconf -i out -o out
./exttpose -i out -o out -s 0 -l -x
rm $fileinput
cp out.data out.conf out.idx out.tpose ../cSPADE
cd ../cSPADE

comm="./spade -e 1 -r -i out -s $minsup"

output=../results/$result
# HEADER configuration of this section
echo -n "Time: " > $output
date -R >> $output
echo "Input file: $fileinput" >> $output
echo >> $output
echo "Parameters: " >> $output
echo "Minsup: $minsup" >> $output 

echo "Output file:$output"

# creation of spade command with options
if [ -n "$maxgap" ]
then
	comm=$comm" -u $maxgap" 
	echo "Maxgap: $maxgap" >> $output
fi
if [ -n "$mingap" ]
then
	comm=$comm" -l $mingap"
	echo "Mingap: $mingap" >> $output
fi
if [ -n "$maxwindow" ]
then
	comm=$comm" -w $maxwindow"
	echo "Maxwindow: $maxwindow" >> $output
fi
echo
echo >> $output

comm=$comm" -o" 
$comm >> $output

#gedit $output
